/*****************************************************************
*  sthp.c  this sources file functions that is implemented String Type 
*          Handle Protocol. 
*  
* Copyright (C) 2009 Andrey Y. Rjavskov(aka rjaan) <rjaan@yandex.ru> 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
*****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdarg.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "sthp.h"

static ssize_t sthp_readn ( int sockfd, void *vptr, size_t n );
static ssize_t sthp_writedn ( int sockfd, void *vptr, size_t n );

char* 
sthp_puts( void ) 
{
  int ch;  
  char *str_in = NULL; 

  while ( 1 ) {
     int nchars;
     char *ptr;
     ch=getc(stdin);
   
     if ( ch == EOF || ch == '\n') {
        if ( str_in == NULL ) { 
         return  strdup((const char*)EMPETY_STRING); 
        }
        return str_in; 
     }
     if ( str_in == NULL ) {
        str_in = malloc(2*sizeof(char)); 
        *str_in = (char)ch; 
        *(str_in+1) = '\0';
        continue;
     }
     ptr = str_in;
     nchars = strlen ( ptr );
     str_in = realloc ( ptr, nchars * sizeof ( char ) + 2 ); 
     *( str_in + nchars ) = (char)ch; 
     *( str_in + nchars + 1 ) = '\0';
  }

  return str_in;

}

void print_stderr ( int nchars, const char* format, ... )
{

	struct timeval   tv;
	struct tm*       tm;
	va_list          args;
	char         *strftime = malloc ( 64 + nchars );

        fputc ( 0x20, stderr );	
	gettimeofday(&tv,NULL);
	tm = localtime( &tv.tv_sec);

	sprintf( strftime,"%04d-%02d-%02d %02d:%02d:%02d.%03ld PID %d ",
					1900+tm->tm_year, ++tm->tm_mon, tm->tm_mday,
					tm->tm_hour, tm->tm_min, tm->tm_sec,
					tv.tv_usec/1000, getpid() );
	fputs ( strftime, stderr );
        fputc ( 0x20, stderr );	
	va_start(args, format);
	vfprintf(stderr, format, args);
	va_end(args);
	fflush(stderr);
	free(strftime);
}

void 
sthp_free( char *s )
{
  if ( !s ) return;
  free(s);
}

StringTypeId 
sthp_is_string_type_from( const char *str )
{
   char *ptr = (char*)str;
   if ( !str || !strlen ( str ) ) {
      return STI_EMPTY;
   }
   for ( ; ptr && *ptr; ptr++ ) {
	if( !isdigit ( *ptr ) ) {
	  return STI_CHARS;
	}
   } 
   return STI_DEC;
}  

static int   *_sockfd_ptr;
static void sig_handler ( int sig_num );

void sthp_set_signal (  int *sockfd_ptr ) 
{

  _sockfd_ptr = sockfd_ptr;  

  signal ( SIGINT, sig_handler );
  signal ( SIGTERM, sig_handler );
  signal ( SIGKILL, sig_handler );

}

static void sig_handler ( int sig_num )
{
   int status;
   waitpid ( -1, &status, WNOHANG );
  
   if( _sockfd_ptr != NULL ) { 
     shutdown ( *_sockfd_ptr, 1 );
   }

   print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,"All connections shutdown.Exit of out!\n");

   exit(0); 
}

int
sthp_send ( int sockfd, char* string )
{

   int               pkg_len = strlen ( string ); 
   char              reply_ok [STHP_OK_REPLY_SIZE+1] = { 0 };

   if ( !string ) { return -1; }
   if ( pkg_len <= 0 ) { pkg_len = 1; } 

   if ( sthp_writedn ( sockfd, &pkg_len, sizeof(int) ) == -EXIT_FAILURE ) {
	print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,
		   "can't send a message to server(%s)\n", strerror(errno));
	return -EXIT_FAILURE;
   }

   if ( sthp_readn ( sockfd, reply_ok, STHP_OK_REPLY_SIZE ) == -EXIT_FAILURE ) {
       print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,
		   "can't recv a message from server(%s)\n", strerror(errno));
       return -EXIT_FAILURE;
   }    
   
   if ( sthp_writedn ( sockfd, string, pkg_len ) == -EXIT_FAILURE ) {
	print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,
		   "can't send a message to server(%s)\n", strerror(errno));
	return -EXIT_FAILURE;
   }

   return pkg_len;
}


int 
sthp_recv ( int sockfd, char** string )
{

   int  pkg_len = 0;

   char *sockbufer = NULL;  
    
   if ( sthp_readn  ( sockfd, &pkg_len, sizeof(int) ) == -EXIT_FAILURE ) {
       print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,
		   "can't recv a message from server(%s)\n", strerror(errno) );
       return -EXIT_FAILURE;
   }

   if ( sthp_writedn ( sockfd, STHP_OK_REPLY, STHP_OK_REPLY_SIZE ) == -EXIT_FAILURE ) {
	print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,
		        "can't send a message to server(%s)\n", strerror(errno) );
	return -EXIT_FAILURE;
   }
  
   sockbufer = calloc ( 1, pkg_len + 1);

   if ( sthp_readn  ( sockfd, sockbufer, pkg_len ) == -EXIT_FAILURE){
       print_stderr ( STDERRMSG_HAVE_BYTE_DEFAULT,
		   "can't recv a message from server(%s)\n", strerror(errno));
       return -EXIT_FAILURE;
   }

   *string = sockbufer;

   return pkg_len; 

}

static ssize_t 
sthp_readn ( int sockfd, void *vptr, size_t n )
{
   size_t     nleft;
   ssize_t    nread;
   char       *ptr;

   ptr = vptr;
   nleft = n;
 
   while ( nleft > 0 ) {
     if ( ( nread = read ( sockfd, ptr, nleft ) ) == -EXIT_FAILURE ) {
         if ( errno != EINTR ) { return -EXIT_FAILURE; }
     }
     nleft -= nread;
     ptr   += nread;
     continue;
   }
   return n;
}

static ssize_t 
sthp_writedn ( int sockfd, void *vptr, size_t n )
{

   ssize_t    nwritten;
   size_t     nleft;
   char       *ptr;

   nleft = n;
   ptr = vptr;

   while ( nleft > 0 ) {
     if ( ( nwritten = write ( sockfd, ptr, nleft ) ) == -EXIT_FAILURE ) {
         if ( errno != EINTR ) {  return -EXIT_FAILURE; }
     }
     nleft -= nwritten;
     ptr   += nwritten;
   }
   return n;
}

/*eof*/

